function cross_validation(config_file, datapath, d, a, b, options, seed, cv_options)

addpath(genpath('simpleNN'));

% Read options given by users
param = struct;
if ~isempty(cv_options)
  param = parse_options(param, cv_options);
  param = parse_options(param, options);
else
  error('``cv_options" cannot be empty.');
end
if ~isfield(param, 'lr')
  error('``lr" option must be provided in the sixth argument.');
end

% Read train data sets
[y, Z] = libsvmread(datapath);
y = y - min(y) + 1;
Z = [full(Z) zeros(size(Z,1), a*b*d - size(Z,2))];

% Rearrange data from row-wise to col-wise
Z = reshape(permute(reshape(Z, [],b,a,d), [1,3,2,4]), [], a*b*d);

% Max-min normalization
tmp_max = max(Z, [], 2);
tmp_min = min(Z, [], 2);
Z = (Z - tmp_min) ./ (tmp_max - tmp_min);

% Zero mean
mean_tr = mean(Z);
Z = Z - mean_tr;

% Split training data into cv_splits sets by stratified selection.
l = length(y);
if param.cv_splits > l
	error('The ``cv_splits" option must be less than the # training data.');
end
split_idx = stratified_selection(y, param.cv_splits);

if isfield(param, 'cv_id')
	cv_id = param.cv_id;
	disp(sprintf('CV_id: %d', cv_id));

	[num_correct, num_subset] = do_cross_validation(y, Z, config_file, options, seed, split_idx, cv_id);
	disp(sprintf('config: %s CV_id: %d lr: %f num_correct: %d num_subset: %d\n', ...
                     config_file, cv_id, param.lr, num_correct, num_subset));

	if isfield(param, 'output_path')
		fid = fopen(param.output_path, 'a');
		fprintf(fid, 'config: %s CV_id: %d lr: %f num_correct: %d num_subset: %d\n', ...
			config_file, cv_id, param.lr, num_correct, num_subset);
		fclose(fid);
	end
else
	total_correct = 0;
	for cv_id = 1 : param.cv_splits
		disp(sprintf('CV_id: %d', cv_id));

		[num_correct, ~] = do_cross_validation(y, Z, config_file, options, seed, split_idx, cv_id);

		total_correct = total_correct + num_correct;
	end
	disp(sprintf('config: %s lr: %f total_correct: %d/%d acc: %f\n', ...
                     config_file, param.lr, total_correct, l, total_correct/l));

	if isfield(param, 'output_path')
		fid = fopen(param.output_path, 'a');
		fprintf(fid, 'config: %s lr: %f total_correct: %d/%d acc: %f\n', ...
			config_file, param.lr, total_correct, l, total_correct/l);
		fclose(fid);
	end
end
%end
%[v, k] = max(num_correct);
%msg = sprintf('selected_lr: %f', lr_candidates(k));
%disp(msg);


function [num_correct, num_subset] = do_cross_validation(y, Z, config_file, options, seed, split_idx, cv_id)

l = length(y);
train_idx = setdiff(1:l, split_idx{cv_id});
model = cnn_train(y(train_idx), Z(train_idx, :), config_file, options, seed);
y_valid = y(split_idx{cv_id});
[predicted_label, acc] = cnn_predict(y_valid, Z(split_idx{cv_id}, :), model);

num_correct = sum(predicted_label == y_valid);
num_subset = length(y_valid);


function split_idx = stratified_selection(y, cv_splits)
% Output: cell matrix with shape of (1, cv_splits), each cell consists of the indices of instance by stratified selection.

s = RandStream('mt19937ar','Seed',0);

K = length(unique(y));
split_idx = cell(1, cv_splits);
instances_k = cell(K, 1);
remain_idx = [];

for k = 1 : K
	% Find the indices of class k
	instances_k{k} = find(y == k);

  % Random shuffle
  instances_k{k} = instances_k{k}(randperm(s, length(instances_k{k})));

	% Calculate the least number of instances of class k in each CV group
	num_sampled = floor(length(instances_k{k}) / cv_splits);
	if num_sampled == 0
		error('The instances with class %d cannot be distributed to each CV group because of insufficient number.', k);
	end
	% Assign the index of instance into each CV group
	split_idx = arrayfun(@(cv_id) vertcat(split_idx{cv_id}, instances_k{k}((cv_id-1)*num_sampled+1:cv_id*num_sampled)), 1:cv_splits, 'UniformOutput', 0);
	% Collect the remaining index of instances
	remain_idx = vertcat(remain_idx, instances_k{k}(cv_splits*num_sampled+1:end));
end

% Assign the index of remaining instance into each CV group in order
num_sampled = floor(length(remain_idx) / cv_splits);

for cv_id = 1 : cv_splits-1
	split_idx{cv_id} = vertcat(split_idx{cv_id}, remain_idx((cv_id-1)*num_sampled+1:cv_id*num_sampled));
end
split_idx{cv_splits} = vertcat(split_idx{cv_splits}, remain_idx(cv_id*num_sampled+1:end));


function param = parse_options(param, options)

options = strsplit(strtrim(options), ' ');

if mod(length(options), 2) == 1
  error('Each option is specified by its name and value.');
end

for i = 1 : length(options)/2
  option = options{2*i-1};
  value = options{2*i};
  switch option
    case '-v'
      param.cv_splits = str2num(value);
    case '-cv_id'
      param.cv_id = str2num(value);
    case '-output'
      param.output_path = value;
    case '-lr'
      param.lr = str2num(value);
    %otherwise
    %  error('%s is not a supported option.', option);
  end
end
