#!/usr/bin/python
import argparse
import re
import sys
from collections import defaultdict

dataset = ['MNIST', 'SVHN', 'CIFAR10', 'smallNORB']
config_map = {'MNIST': {'layer3': '../config/mnist-layer3.config',
                        'layer5': '../config/mnist-layer5.config'},
              'SVHN': {'layer3': '../config/SVHN-layer3.config',
                       'layer5': '../config/SVHN-layer5.config'},
              'CIFAR10': {'layer3': '../config/cifar10-layer3.config',
                          'layer5': '../config/cifar10-layer5.config'},
              'smallNORB': {'layer3': '../config/smallNORB-32x32-layer3.config',
                            'layer5': '../config/smallNORB-32x32-layer5.config'}}
num_seed = 5

def get_CV_results(in_filename, cv_splits):
    """ Gather results
    """
    num_corrects = {}
    num_instances = {}
    cv = {}
    with open(in_filename, 'r') as fp:
        for line in fp:
            if re.match(r'^config: ', line):
                result = line.split()
                config, cv_id, lr, num_correct, num_subset = result[1], \
                    int(result[3]), float(result[5]), int(result[7]), int(result[9])

                if not config in cv:
                    cv[config] = defaultdict(int)
                    num_corrects[config] = defaultdict(int)
                    num_instances[config] = defaultdict(int)
                cv[config][lr] += 1
                num_corrects[config][lr] += num_correct
                num_instances[config][lr] += num_subset

    best_lr = defaultdict(int)
    for config, dict_ in num_corrects.items():
        if all([cv[config][lr] == cv_splits for lr in dict_]):
            tmp = 0
            tmp_lr = 0
            for lr in dict_:
                if num_corrects[config][lr] > tmp:
                    tmp = num_corrects[config][lr]
                    tmp_lr = lr
            best_lr[config] = tmp_lr
        else:
            print(
                ('The result of cross-validation for %s model is not complete.'
                 % config))
    return best_lr, num_corrects, num_instances


def print_CV_results(best_lr, num_corrects, num_instances, out_filename=None):
    try:
        if out_filename:
            fp = open(out_filename, 'w')
        else:
            fp = sys.stdout

        for config in best_lr:
            lr = best_lr[config]
            print(
                'config: {0} best lr: {1} num_corrects: {2} num_instances: {3}'
                .format(config, lr, num_corrects[config][lr],
                        num_instances[config][lr]), file=fp)
    except IOError as e:
        print('Error:', e)
    finally:
        if out_filename:
            fp.close()

def get_exp1_newton_results(in_filename, res={}):
    """ Gather results
    """
    with open(in_filename, 'r') as fp:
        for line in fp:
            if re.match(r'^config: ', line):
                result = line.split()
                config, seed, test_acc, test_prec, test_recall, test_f1 = (
                    result[1], int(result[3]), float(result[5]), float(result[7]),
                    float(result[9]), float(result[11]))

                if 'newton' not in res:
                    res['newton'] = {
                        config: {
                            'seed': {
                                seed: {
                                    'test_acc': test_acc,
                                    'test_prec': test_prec,
                                    'test_recall': test_recall,
                                    'test_f1': test_f1,
                                }
                            }
                        }
                    }
                elif config not in res['newton']:
                    res['newton'].update({
                        config: {
                            'seed': {
                                seed: {
                                    'test_acc': test_acc,
                                    'test_prec': test_prec,
                                    'test_recall': test_recall,
                                    'test_f1': test_f1,
                                }
                            }
                        }
                    })
                elif config in res['newton']:
                    res['newton'][config]['seed'].update({
                        seed: {
                            'test_acc': test_acc,
                            'test_prec': test_prec,
                            'test_recall': test_recall,
                            'test_f1': test_f1,
                        }
                    })

    for config, dict_ in res['newton'].items():
        if len(res['newton'][config]['seed']) == num_seed:
            for seed in res['newton'][config]['seed']:
                res['newton'][config]['avg_acc'] = res['newton'][config].get('avg_acc', 0) + res['newton'][config]['seed'][seed]['test_acc']
                res['newton'][config]['avg_prec'] = res['newton'][config].get('avg_prec', 0) + res['newton'][config]['seed'][seed]['test_prec']
                res['newton'][config]['avg_recall'] = res['newton'][config].get('avg_recall', 0) + res['newton'][config]['seed'][seed]['test_recall']
                res['newton'][config]['avg_f1'] = res['newton'][config].get('avg_f1', 0) + res['newton'][config]['seed'][seed]['test_f1']
            res['newton'][config]['avg_acc'] /= num_seed * 0.01
            res['newton'][config]['avg_prec'] /= num_seed * 0.01
            res['newton'][config]['avg_recall'] /= num_seed * 0.01
            res['newton'][config]['avg_f1'] /= num_seed * 0.01
        else:
            print(
                ('The newton result for %s model is not complete.' % config))

    return res

def get_exp1_sg_results(in_filename, res={}):
    """ Gather results
    """
    with open(in_filename, 'r') as fp:
        for line in fp:
            if re.match(r'^config: ', line):
                result = line.split()
                config, seed, lr, test_acc, test_prec, test_recall, test_f1 = (
                    result[1], int(result[3]), float(result[5]), float(result[7]),
                    float(result[9]), float(result[11]), float(result[13]))

                if 'sg' not in res:
                    res['sg'] = {
                        config: {
                            'lr': lr,
                            'seed': {
                                seed: {
                                    'test_acc': test_acc,
                                    'test_prec': test_prec,
                                    'test_recall': test_recall,
                                    'test_f1': test_f1,
                                }
                            }
                        }
                    }
                elif config not in res['sg']:
                    res['sg'].update({
                        config: {
                            'lr': lr,
                            'seed': {
                                seed: {
                                    'test_acc': test_acc,
                                    'test_prec': test_prec,
                                    'test_recall': test_recall,
                                    'test_f1': test_f1,
                                }
                            }
                        }
                    })
                elif config in res['sg']:
                    res['sg'][config]['seed'].update({
                        seed: {
                            'test_acc': test_acc,
                            'test_prec': test_prec,
                            'test_recall': test_recall,
                            'test_f1': test_f1,
                        }
                    })

    for config, dict_ in res['sg'].items():
        if len(res['sg'][config]['seed']) == num_seed:
            for seed in res['sg'][config]['seed']:
                res['sg'][config]['avg_acc'] = res['sg'][config].get('avg_acc', 0) + res['sg'][config]['seed'][seed]['test_acc']
                res['sg'][config]['avg_prec'] = res['sg'][config].get('avg_prec', 0) + res['sg'][config]['seed'][seed]['test_prec']
                res['sg'][config]['avg_recall'] = res['sg'][config].get('avg_recall', 0) + res['sg'][config]['seed'][seed]['test_recall']
                res['sg'][config]['avg_f1'] = res['sg'][config].get('avg_f1', 0) + res['sg'][config]['seed'][seed]['test_f1']
            res['sg'][config]['avg_acc'] /= num_seed * 0.01
            res['sg'][config]['avg_prec'] /= num_seed * 0.01
            res['sg'][config]['avg_recall'] /= num_seed * 0.01
            res['sg'][config]['avg_f1'] /= num_seed * 0.01
        else:
            print(
                ('The sg result for %s model is not complete.' % config))

    return res

def print_exp1_results(res, out_filename=None):
    """ Generate Table 5
    """
    try:
        if out_filename:
            fp = open(out_filename, 'w')
        else:
            fp = sys.stdout

        print(r'\begin{table}[t]', file=fp)
        print(r'\caption{', file=fp)
        print('Test accuracy by Newton and SG methods. '
              'We use five random seeds and report the mean test accuracy. '
              'The value within the parenthesis is the initial learning rate for SG, selected from '
              'a cross validation procedure on the training set.}', file=fp)
        print(r'\label{table:exps-results-CNN}', file=fp)
        print(r'\begin{center}', file=fp)
        print(r'\begin{tabular}{lclcl}', file=fp)
        print(r'\multicolumn{1}{c}{} & \multicolumn{2}{c}{3-layer CNN} & '
              r'\multicolumn{2}{c}{5-layer CNN} \\', file=fp)
        print(r'\hline', file=fp)
        print(r' & \multicolumn{1}{c}{Newton} & \multicolumn{1}{c}{SG} & '
              r'\multicolumn{1}{c}{Newton} & \multicolumn{1}{c}{SG} \\', file=fp)
        for dataset_name in dataset:
            print(r'\hline', file=fp)
            print(r'{\sf %s} & $%.2f\%%$ & $%.2f\%%$ (%g) & $%.2f\%%$ & '
                  r'$%.2f\%%$ (%g) \\' % (
                      dataset_name,
                      res.get(
                          'newton', {}).get(
                              config_map[dataset_name]['layer3'], {}).get(
                                  'avg_acc', 0),
                      res.get(
                          'sg', {}).get(
                              config_map[dataset_name]['layer3'], {}).get(
                                  'avg_acc', 0),
                      res.get(
                          'sg', {}).get(
                              config_map[dataset_name]['layer3'], {}).get(
                                  'lr', 0),
                      res.get(
                          'newton', {}).get(
                              config_map[dataset_name]['layer5'], {}).get(
                                  'avg_acc', 0),
                      res.get(
                          'sg', {}).get(
                              config_map[dataset_name]['layer5'], {}).get(
                                  'avg_acc', 0),
                      res.get(
                          'sg', {}).get(
                              config_map[dataset_name]['layer5'], {}).get(
                                  'lr', 0),
                      ), file=fp)
        print(r'\end{tabular}', file=fp)
        print(r'\end{center}', file=fp)
        print(r'\end{table}', file=fp)
    except IOError as e:
        print('Error:', e)
    finally:
        if out_filename:
            fp.close()


def print_exp1_ancillary_results(res, out_filename=None):
    """ Generate Table B
    """
    try:
        if out_filename:
            fp = open(out_filename, 'w')
        else:
            fp = sys.stdout

        print(r'\begin{table}[t]', file=fp)
        print(r'\caption{', file=fp)
        print(r'Results of considering some statistic metrics other than '
            r'accuracy. All settings are the same as those for Table '
            r'\ref{table:exps-results-CNN} in the paper.}', file=fp)
        print(r'\label{table:exps-macro-average-metrics}', file=fp)
        print(r'\begin{center}', file=fp)
        print(r'\begin{tabular}{l|ccc|ccc}', file=fp)
        print(r'\multicolumn{1}{c}{} & \multicolumn{3}{c}{Newton} & '
            r'\multicolumn{3}{c}{SG} \\', file=fp)
        print(r'\hline')
        print(r'Data set & Precision & Recall & F1 score & Precision & Recall & '
            r'F1 score \\')
        for dataset_name in dataset:
            print(r'\hline')
            print(r'{\sf %s} & $%.1f\%%$ & $%.1f\%%$ & $%.1f\%%$ & $%.1f\%%$ & '
                r'$%.1f\%%$ & $%.1f\%%$  \\' % (
                    dataset_name,
                    res.get(
                        'newton', {}).get(
                            config_map[dataset_name]['layer3'], {}).get(
                                'avg_prec', 0),
                    res.get(
                        'newton', {}).get(
                            config_map[dataset_name]['layer3'], {}).get(
                                'avg_recall', 0),
                    res.get(
                        'newton', {}).get(
                            config_map[dataset_name]['layer3'], {}).get(
                                'avg_f1', 0),
                    res.get(
                        'sg', {}).get(
                            config_map[dataset_name]['layer3'], {}).get(
                                'avg_prec', 0),
                    res.get(
                        'sg', {}).get(
                            config_map[dataset_name]['layer3'], {}).get(
                                'avg_recall', 0),
                    res.get(
                        'sg', {}).get(
                            config_map[dataset_name]['layer3'], {}).get(
                                'avg_f1', 0),
                    ), file=fp)
        print(r'\end{tabular}', file=fp)
        print(r'\subcaption{3-layer CNN.}', file=fp)
        print(r'\end{center}', file=fp)
        print(r'\begin{center}', file=fp)
        print(r'\begin{tabular}{l|ccc|ccc}', file=fp)
        print(r'\multicolumn{1}{c}{} & \multicolumn{3}{c}{Newton} & '
            r'\multicolumn{3}{c}{SG} \\', file=fp)
        print(r'\hline')
        print(r'Data set & Precision & Recall & F1 score & Precision & Recall & '
            r'F1 score \\')
        for dataset_name in dataset:
            print(r'\hline')
            print(r'{\sf %s} & $%.1f\%%$ & $%.1f\%%$ & $%.1f\%%$ & $%.1f\%%$ & '
                r'$%.1f\%%$ & $%.1f\%%$  \\' % (
                    dataset_name,
                    res.get(
                        'newton', {}).get(
                            config_map[dataset_name]['layer5'], {}).get(
                                'avg_prec', 0),
                    res.get(
                        'newton', {}).get(
                            config_map[dataset_name]['layer5'], {}).get(
                                'avg_recall', 0),
                    res.get(
                        'newton', {}).get(
                            config_map[dataset_name]['layer5'], {}).get(
                                'avg_f1', 0),
                    res.get(
                        'sg', {}).get(
                            config_map[dataset_name]['layer5'], {}).get(
                                'avg_prec', 0),
                    res.get(
                        'sg', {}).get(
                            config_map[dataset_name]['layer5'], {}).get(
                                'avg_recall', 0),
                    res.get(
                        'sg', {}).get(
                            config_map[dataset_name]['layer5'], {}).get(
                                'avg_f1', 0),
                    ), file=fp)
        print(r'\end{tabular}', file=fp)
        print(r'\subcaption{5-layer CNN.}', file=fp)
        print(r'\end{center}', file=fp)
        print(r'\end{table}', file=fp)

    except IOError as e:
        print('Error:', e)
    finally:
        if out_filename:
            fp.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Dispatcher')
    parser.add_argument(
        '--in_filename',
        type=str,
        metavar='raw results file name',
        required=True,
        help='Specify the filename storing the results from each worker')
    parser.add_argument('--cv_splits',
                        type=int,
                        metavar='# CV folds',
                        required=True,
                        help='Specify the # CV folds')
    parser.add_argument('--out_filename',
                        type=str,
                        metavar='output file name',
                        help='Specify the output file name')
    args = parser.parse_args()

    best_lr, num_corrects, num_instances = get_CV_results(
        args.in_filename, args.cv_splits)
    print_CV_results(best_lr, num_corrects, num_instances, args.out_filename)
